@@ -6,7 +6,7 @@ use 5.014;
 
 no if $] >= 5.018, warnings => 'experimental::smartmatch';
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
 
 use Carp qw(confess cluck);
 use DateTime;
@@ -217,7 +217,7 @@ sub get_realtime {
 		}
 
 		if ( not $self->{serializable} ) {
-			$result->add_realtime($s);
+			$result->set_realtime($s);
 		}
 
 		for my $e_m (@e_ms) {
@@ -233,10 +233,10 @@ sub get_realtime {
 			}
 		}
 
-		$result->add_messages(%messages);
+		$result->set_messages(%messages);
 
 		if ($e_tl) {
-			$result->add_tl(
+			$result->set_tl(
 				class     => $e_tl->getAttribute('f'),    # D N S F
 				unknown_t => $e_tl->getAttribute('t'),    # p
 				train_no  => $e_tl->getAttribute('n'),    # dep number
@@ -246,7 +246,7 @@ sub get_realtime {
 			);
 		}
 		if ($e_ref) {
-			$result->add_ref(
+			$result->set_ref(
 				class     => $e_ref->getAttribute('f'),    # D N S F
 				unknown_t => $e_ref->getAttribute('t'),    # p
 				train_no  => $e_ref->getAttribute('n'),    # dep number
@@ -256,7 +256,7 @@ sub get_realtime {
 			);
 		}
 		if ($e_ar) {
-			$result->add_ar(
+			$result->set_ar(
 				arrival_ts      => $e_ar->getAttribute('ct'),
 				plan_arrivaL_ts => $e_ar->getAttribute('pt'),
 				platform        => $e_ar->getAttribute('cp'),
@@ -267,7 +267,7 @@ sub get_realtime {
 			);
 		}
 		if ($e_dp) {
-			$result->add_dp(
+			$result->set_dp(
 				departure_ts      => $e_dp->getAttribute('ct'),
 				plan_departure_ts => $e_dp->getAttribute('pt'),
 				platform          => $e_dp->getAttribute('cp'),
@@ -327,7 +327,7 @@ Travel::Status::DE::IRIS - Interface to IRIS based web departure monitors.
 
 =head1 VERSION
 
-version 0.07
+version 0.08
 
 =head1 DESCRIPTION
 
