@@ -14,7 +14,84 @@ use DateTime::Format::Strptime;
 use List::Compare;
 use List::MoreUtils qw(none uniq firstval);
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
+
+my %translation = (
+	2  => 'Polizeiliche Ermittlung',
+	3  => 'Feuerwehreinsatz neben der Strecke',
+	5  => 'Ärztliche Versorgung eines Fahrgastes',
+	6  => 'Betätigen der Notbremse',
+	7  => 'Personen im Gleis',
+	8  => 'Notarzteinsatz am Gleis',
+	9  => 'Streikauswirkungen',
+	10 => 'Ausgebrochene Tiere im Gleis',
+	11 => 'Unwetter',
+	13 => 'Pass- und Zollkontrolle',
+	15 => 'Beeinträchtigung durch Vandalismus',
+	16 => 'Entschärfung einer Fliegerbombe',
+	17 => 'Beschädigung einer Brücke',
+	18 => 'Umgestürzter Baum im Gleis',
+	19 => 'Unfall an einem Bahnübergang',
+	20 => 'Tiere im Gleis',
+	21 => 'Warten auf weitere Reisende',
+	22 => 'Witterungsbedingte Störung',
+	23 => 'Feuerwehreinsatz auf Bahngelände',
+	24 => 'Verspätung aus dem Ausland',
+	25 => 'Warten auf verspätete Zugteile',
+	28 => 'Gegenstände im Gleis',
+	31 => 'Bauarbeiten',
+	32 => 'Verzögerung beim Ein-/Ausstieg',
+	33 => 'Oberleitungsstörung',
+	34 => 'Signalstörung',
+	35 => 'Streckensperrung',
+	36 => 'Technische Störung am Zug',
+	38 => 'Technische Störung an der Strecke',
+	39 => 'Anhängen von zusätzlichen Wagen',
+	40 => 'Stellwerksstörung/-ausfall',
+	41 => 'Störung an einem Bahnübergang',
+	42 => 'Außerplanmäßige Geschwindigkeitsbeschränkung',
+	43 => 'Verspätung eines vorausfahrenden Zuges',
+	44 => 'Warten auf einen entgegenkommenden Zug',
+	45 => 'Überholung durch anderen Zug',
+	46 => 'Warten auf freie Einfahrt',
+	47 => 'Verspätete Bereitstellung',
+	48 => 'Verspätung aus vorheriger Fahrt',
+	55 => 'Technische Störung an einem anderen Zug',           # ?
+	56 => 'Warten auf Fahrgäste aus einem Bus',
+	57 => 'Zusätzlicher Halt',
+	58 => 'Umleitung',                                          # ?
+	59 => 'Schnee und Eis',
+	60 => 'Reduzierte Geschwindigkeit wegen Sturm(?)',
+	61 => 'Türstörung',
+	62 => 'Behobene technische Störung am Zug',
+	63 => 'Technische Untersuchung am Zug',
+	64 => 'Weichenstörung',
+	65 => 'Erdrutsch',
+	73 => 'Mehrzweckabteil vorne',
+	74 => 'Mehrzweckabteil hinten',
+	75 => '1. Klasse vorne',
+	76 => '1. Klasse hinten',
+	77 => 'Ohne 1. Klasse',
+	80 => 'Abweichende Wagenreihung',
+	82 => 'Mehrere Wagen fehlen',
+	83 => 'Fehlender Zugteil',
+	84 => 'Zug verkehrt richtig gereiht',                       # r 80 82 83 85
+	85 => 'Ein Wagen fehlt',
+	86 => 'Keine Reservierungsanzeige',
+	87 => 'Einzelne Wagen ohne Reservierungsanzeige',
+	88 => 'Keine Qualitätsmängel',  # r 80 82 83 85 86 87 90 91 92 93 96 97 98
+	89 => 'Reservierungen sind wieder vorhanden',
+	90 => 'Kein Bordrestaurant/Bordbistro',
+	91 => 'Eingeschränkte Fahrradmitnahme',
+	92 => 'Klimaanlage in einzelnen Wagen ausgefallen',
+	93 => 'Fehlende oder gestörte behindertengerechte Einrichtung',
+	94 => 'Ersatzbewirtschaftung',
+	95 => 'Ohne behindertengerechtes WC',
+	96 => 'Der Zug ist überbesetzt',
+	97 => 'Der Zug ist überbesetzt',
+	98 => 'Sonstige Qualitätsmängel',
+	99 => 'Verzögerungen im Betriebsablauf',
+);
 
 Travel::Status::DE::IRIS::Result->mk_ro_accessors(
 	qw(arrival classes date datetime delay departure is_cancelled is_transfer
@@ -94,7 +171,7 @@ sub new {
 	return bless( $ref, $obj );
 }
 
-sub add_ar {
+sub set_ar {
 	my ( $self, %attrib ) = @_;
 
 	my $strp = DateTime::Format::Strptime->new(
@@ -115,16 +192,28 @@ sub add_ar {
 		  = $self->arrival->subtract_datetime( $self->sched_arrival )
 		  ->in_units('minutes');
 	}
+	else {
+		$self->{arrival} = $self->{sched_arrival};
+		$self->{delay}   = 0;
+	}
 
 	if ( $attrib{platform} ) {
 		$self->{platform} = $attrib{platform};
 	}
+	else {
+		$self->{platform} = $self->{sched_platform};
+	}
 
 	if ( $attrib{route_pre} ) {
 		$self->{route_pre} = [ split( qr{[|]}, $attrib{route_pre} // q{} ) ];
 		$self->{route_start} = $self->{route_pre}[0];
 	}
+	else {
+		$self->{route_pre}   = $self->{sched_route_pre};
+		$self->{route_start} = $self->{sched_route_start};
+	}
 
+	# also only for unscheduled arrivals
 	if ( $attrib{sched_route_pre} ) {
 		$self->{sched_route_pre}
 		  = [ split( qr{[|]}, $attrib{sched_route_pre} // q{} ) ];
@@ -134,11 +223,14 @@ sub add_ar {
 	if ( $attrib{status} and $attrib{status} eq 'c' ) {
 		$self->{is_cancelled} = 1;
 	}
+	else {
+		$self->{is_cancelled} = 0;
+	}
 
 	return $self;
 }
 
-sub add_dp {
+sub set_dp {
 	my ( $self, %attrib ) = @_;
 
 	my $strp = DateTime::Format::Strptime->new(
@@ -159,16 +251,28 @@ sub add_dp {
 		  = $self->departure->subtract_datetime( $self->sched_departure )
 		  ->in_units('minutes');
 	}
+	else {
+		$self->{departure} = $self->{sched_departure};
+		$self->{delay}     = 0;
+	}
 
 	if ( $attrib{platform} ) {
 		$self->{platform} = $attrib{platform};
 	}
+	else {
+		$self->{platform} = $self->{sched_platform};
+	}
 
 	if ( $attrib{route_post} ) {
 		$self->{route_post} = [ split( qr{[|]}, $attrib{route_post} // q{} ) ];
 		$self->{route_end} = $self->{route_post}[-1];
 	}
+	else {
+		$self->{route_post} = $self->{sched_route_post};
+		$self->{route_end}  = $self->{sched_route_end};
+	}
 
+	# also only for unscheduled departures
 	if ( $attrib{sched_route_post} ) {
 		$self->{sched_route_post}
 		  = [ split( qr{[|]}, $attrib{sched_route_post} // q{} ) ];
@@ -178,11 +282,14 @@ sub add_dp {
 	if ( $attrib{status} and $attrib{status} eq 'c' ) {
 		$self->{is_cancelled} = 1;
 	}
+	else {
+		$self->{is_cancelled} = 0;
+	}
 
 	return $self;
 }
 
-sub add_messages {
+sub set_messages {
 	my ( $self, %messages ) = @_;
 
 	$self->{messages} = \%messages;
@@ -190,7 +297,7 @@ sub add_messages {
 	return $self;
 }
 
-sub add_realtime {
+sub set_realtime {
 	my ( $self, $xmlobj ) = @_;
 
 	$self->{realtime_xml} = $xmlobj;
@@ -198,7 +305,7 @@ sub add_realtime {
 	return $self;
 }
 
-sub add_ref {
+sub set_ref {
 	my ( $self, %attrib ) = @_;
 
 	$self->{train_no_transfer} = $attrib{train_no};
@@ -208,7 +315,7 @@ sub add_ref {
 	return $self;
 }
 
-sub add_tl {
+sub set_tl {
 	my ( $self, %attrib ) = @_;
 
 	# TODO
@@ -327,6 +434,12 @@ sub delay_messages {
 	return @ret;
 }
 
+sub dump_message_codes {
+	my ($self) = @_;
+
+	return %translation;
+}
+
 sub qos_messages {
 	my ($self) = @_;
 
@@ -336,7 +449,8 @@ sub qos_messages {
 	);
 
 	my @keys = sort keys %{ $self->{messages} };
-	my @msgs = grep { $_->[1] ~~ [qw[f q]] } map { $self->{messages}{$_} } @keys;
+	my @msgs
+	  = grep { $_->[1] ~~ [qw[f q]] } map { $self->{messages}{$_} } @keys;
 	my @ret;
 
 	for my $msg (@msgs) {
@@ -359,6 +473,25 @@ sub qos_messages {
 	return @ret;
 }
 
+sub raw_messages {
+	my ($self) = @_;
+
+	my $strp = DateTime::Format::Strptime->new(
+		pattern   => '%y%m%d%H%M',
+		time_zone => 'Europe/Berlin',
+	);
+
+	my @messages = reverse sort keys %{ $self->{messages} };
+	my @ret      = map {
+		[
+			$strp->parse_datetime( $self->{messages}->{$_}->[0] ),
+			$self->{messages}->{$_}->[2]
+		]
+	} @messages;
+
+	return @ret;
+}
+
 sub messages {
 	my ($self) = @_;
 
@@ -507,80 +640,6 @@ sub superseded_messages {
 sub translate_msg {
 	my ( $self, $msg ) = @_;
 
-	my %translation = (
-		2  => 'Polizeiliche Ermittlung',
-		3  => 'Feuerwehreinsatz neben der Strecke',
-		5  => 'Ärztliche Versorgung eines Fahrgastes',
-		6  => 'Betätigen der Notbremse',
-		7  => 'Personen im Gleis',
-		8  => 'Notarzteinsatz am Gleis',
-		9  => 'Streikauswirkungen',
-		10 => 'Ausgebrochene Tiere im Gleis',
-		11 => 'Unwetter',
-		15 => 'Beeinträchtigung durch Vandalismus',
-		16 => 'Entschärfung einer Fliegerbombe',
-		17 => 'Beschädigung einer Brücke',
-		18 => 'Umgestürzter Baum im Gleis',
-		19 => 'Unfall an einem Bahnübergang',
-		20 => 'Tiere im Gleis',
-		21 => 'Warten auf weitere Reisende',
-		22 => 'Witterungsbedingte Störung',
-		23 => 'Feuerwehreinsatz auf Bahngelände',
-		24 => 'Verspätung aus dem Ausland',
-		25 => 'Warten auf verspätete Zugteile',
-		28 => 'Gegenstände im Gleis',
-		31 => 'Bauarbeiten',
-		32 => 'Verzögerung beim Ein-/Ausstieg',
-		33 => 'Oberleitungsstörung',
-		34 => 'Signalstörung',
-		35 => 'Streckensperrung',
-		36 => 'Technische Störung am Zug',
-		38 => 'Technische Störung an der Strecke',
-		39 => 'Anhängen von zusätzlichen Wagen',
-		40 => 'Stellwerksstörung/-ausfall',
-		41 => 'Störung an einem Bahnübergang',
-		42 => 'Außerplanmäßige Geschwindigkeitsbeschränkung',
-		43 => 'Verspätung eines vorausfahrenden Zuges',
-		44 => 'Warten auf einen entgegenkommenden Zug',
-		45 => 'Überholung durch anderen Zug',
-		46 => 'Warten auf freie Einfahrt',
-		47 => 'Verspätete Bereitstellung',
-		48 => 'Verspätung aus vorheriger Fahrt',
-		55 => 'Technische Störung an einem anderen Zug',        # ?
-		56 => 'Warten auf Fahrgäste aus einem Bus',
-		57 => 'Zusätzlicher Halt',
-		58 => 'Umleitung',                                       # ?
-		59 => 'Schnee und Eis',
-		61 => 'Türstörung',
-		62 => 'Behobene technische Störung am Zug',
-		63 => 'Technische Untersuchung am Zug',
-		64 => 'Weichenstörung',
-		65 => 'Erdrutsch',
-		73 => 'Mehrzweckabteil vorne',
-		74 => 'Mehrzweckabteil hinten',
-		77 => 'Ohne 1. Klasse',
-		80 => 'Abweichende Wagenreihung',
-		82 => 'Mehrere Wagen fehlen',
-		83 => 'Fehlender Zugteil',
-		84 => 'Zug verkehrt richtig gereiht',                    # r 80 82 83 85
-		85 => 'Ein Wagen fehlt',
-		86 => 'Keine Reservierungsanzeige',
-		87 => 'Einzelne Wagen ohne Reservierungsanzeige',
-		88 =>
-		  'Keine Qualitätsmängel',  # r 80 82 83 85 86 87 90 91 92 93 96 97 98
-		89 => 'Reservierungen sind wieder vorhanden',
-		90 => 'Kein Bordrestaurant/Bordbistro',
-		91 => 'Eingeschränkte Fahrradmitnahme',
-		92 => 'Klimaanlage in einzelnen Wagen ausgefallen',
-		93 => 'Fehlende oder gestörte behindertengerechte Einrichtung',
-		94 => 'Ersatzbewirtschaftung',
-		95 => 'Ohne behindertengerechtes WC',
-		96 => 'Der Zug ist überbesetzt',
-		97 => 'Der Zug ist überbesetzt',
-		98 => 'Sonstige Qualitätsmängel',
-		99 => 'Verzögerungen im Betriebsablauf',
-	);
-
 	return $translation{$msg} // "?($msg)";
 }
 
@@ -613,7 +672,7 @@ arrival/departure received by Travel::Status::DE::IRIS
 
 =head1 VERSION
 
-version 0.07
+version 0.08
 
 =head1 DESCRIPTION
 
@@ -687,7 +746,7 @@ here. Contains realtime data if available.
 
 =item $result->destination
 
-Aleas for route_end.
+Alias for route_end.
 
 =item $result->info
 
@@ -870,8 +929,10 @@ Alias for line.
 
 =item $result->train_id
 
-Numeric ID of this train. Seems to be unique for a year and trackable across
-stations.
+Numeric ID of this train, trackable across stations and days. For instance, the
+S 31128 (S1) to Solingen, starting in Dortmund on 19:23, has the ID
+2404170432985554630 on each station it passes and (usually) on every day of the
+year.  Note that it may change during the yearly itinerary update in december.
 
 =item $result->train_no
 
@@ -934,6 +995,10 @@ Source: Correlation between IRIS and DB RIS (bahn.de).
 
 =item d 11 : "Unwetter"
 
+=item d 13 : "Pass- und Zollkontrolle"
+
+Source: Correlation between IRIS and DB RIS (bahn.de).
+
 =item d 15 : "BeeintrE<auml>chtigung durch Vandalismus"
 
 =item d 16 : "EntschE<auml>rfung einer Fliegerbombe"
@@ -1013,6 +1078,11 @@ to "Notarzteinsatz am Gleis".
 
 Source: Correlation between IRIS and DB RIS (bahn.de).
 
+=item d 60 : "Reduzierte Geschwindigkeit wegen Sturm"
+
+Source: Correlation between IRIS and DB RIS (bahn.de). Needs further
+verification.
+
 =item d 61 : "TE<uuml>rstE<ouml>rung"
 
 Source: Correlation between IRIS and DB RIS (bahn.de).
@@ -1039,6 +1109,14 @@ Source: correlation between IRIS and DB RIS (bahn.de).
 
 Source: correlation between IRIS and DB RIS (bahn.de).
 
+=item q 75 : "1. Klasse vorne"
+
+Source: correlation between IRIS and DB RIS (bahn.de).
+
+=item q 76 : "1. Klasse hinten"
+
+Source: correlation between IRIS and DB RIS (bahn.de).
+
 =item q 77 : "Ohne 1. Klasse"
 
 Source: correlation between IRIS and DB RIS (bahn.de).
